package com.example.pmulabs.api

import com.example.pmulabs.api.model.ArticleRemote
import com.example.pmulabs.api.model.CommentRemote
import com.example.pmulabs.api.model.ReportRemote
import com.example.pmulabs.api.model.TagRemote
import com.example.pmulabs.api.model.UserRemote
import com.jakewharton.retrofit2.converter.kotlinx.serialization.asConverterFactory
import kotlinx.serialization.json.Json
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.http.Body
import retrofit2.http.DELETE
import retrofit2.http.GET
import retrofit2.http.POST
import retrofit2.http.PUT
import retrofit2.http.Path
import retrofit2.http.Query

interface NewsPortalService {
    @GET("report")
    suspend fun getReport(
        @Query("startDate") startDate: Long,
        @Query("endDate") endDate: Long
    ): List<ReportRemote>
    @GET("users")
    suspend fun getUsers(): List<UserRemote>
    @GET("tags")
    suspend fun getTags(
        @Query("_page") page: Int,
        @Query("_limit") limit: Int,
    ): List<TagRemote>
    @GET("tags")
    suspend fun getAllTags(): List<TagRemote>
    @GET("articles")
    suspend fun getAllArticles(): List<ArticleRemote>
    @GET("comments")
    suspend fun getAllComments(): List<CommentRemote>
    @GET("articles")
    suspend fun getArticles(
        @Query("_page") page: Int,
        @Query("_limit") limit: Int,
    ): List<ArticleRemote>
    @GET("comments")
    suspend fun getComments(
        @Query("_page") page: Int,
        @Query("_limit") limit: Int,
    ): List<CommentRemote>

    @GET("comments")
    suspend fun getCountComment(
        @Query("articleId") articleId: Int
    ): List<CommentRemote>
    @GET("comments")
    suspend fun getUserComments(@Query("userId") userId: Int): List<CommentRemote>

    @GET("articles")
    suspend fun getUserArticles(@Query("userId") userId: Int): List<ArticleRemote>

    @GET("tags")
    suspend fun getUserTags(@Query("userId") userId: Int): List<TagRemote>

    @GET("users/{id}")
    suspend fun getUser(
        @Path("id") id: Int,
    ): UserRemote
    @POST("users")
    suspend fun createUser(
        @Body user: UserRemote,
    ): UserRemote
    @PUT("users/{id}")
    suspend fun updateUser(
        @Path("id") id: Int,
        @Body user: UserRemote,
    ): UserRemote
    @DELETE("users/{id}")
    suspend fun deleteUser(
        @Path("id") id: Int,
    ): UserRemote

    @GET("tags")
    suspend fun getTagByName(
        @Query("title") title: String,
    ): List<TagRemote>

    @GET("tags/{id}")
    suspend fun getTag(
        @Path("id") id: Int,
    ): TagRemote
    @POST("tags")
    suspend fun createTag(
        @Body tag: TagRemote,
    ): TagRemote
    @PUT("tags/{id}")
    suspend fun updateTag(
        @Path("id") id: Int,
        @Body tag: TagRemote,
    ): TagRemote
    @DELETE("tags/{id}")
    suspend fun deleteTag(
        @Path("id") id: Int,
    ): TagRemote

    @GET("articles/{id}")
    suspend fun getArticle(
        @Path("id") id: Int,
    ): ArticleRemote
    @POST("articles")
    suspend fun createArticle(
        @Body article: ArticleRemote,
    ): ArticleRemote
    @PUT("articles/{id}")
    suspend fun updateArticle(
        @Path("id") id: Int,
        @Body article: ArticleRemote,
    ): ArticleRemote
    @DELETE("articles/{id}")
    suspend fun deleteArticle(
        @Path("id") id: Int,
    ): ArticleRemote

    @GET("comments/{id}")
    suspend fun getComment(
        @Path("id") id: Int,
    ): CommentRemote
    @POST("comments")
    suspend fun createComment(
        @Body comment: CommentRemote,
    ): CommentRemote
    @PUT("comments/{id}")
    suspend fun updateComment(
        @Path("id") id: Int,
        @Body comment: CommentRemote,
    ): CommentRemote
    @DELETE("comments/{id}")
    suspend fun deleteComment(
        @Path("id") id: Int,
    ): CommentRemote

    companion object {
        private const val BASE_URL = "http://10.0.2.2:8079/"

        @Volatile
        private var INSTANCE: NewsPortalService? = null

        fun getInstance(): NewsPortalService {
            return INSTANCE ?: synchronized(this) {
                val logger = HttpLoggingInterceptor()
                logger.level = HttpLoggingInterceptor.Level.BASIC
                val client = OkHttpClient.Builder()
                    .addInterceptor(logger)
                    .build()
                return Retrofit.Builder()
                    .baseUrl(BASE_URL)
                    .client(client)
                    .addConverterFactory(Json.asConverterFactory("application/json".toMediaType()))
                    .build()
                    .create(NewsPortalService::class.java)
                    .also { INSTANCE = it }
            }
        }
    }
}