package com.example.pmulabs.api.repository

import com.example.pmulabs.api.NewsPortalService
import com.example.pmulabs.api.model.ReportRemote
import com.example.pmulabs.api.model.toArticle
import com.example.pmulabs.api.model.toComment
import com.example.pmulabs.api.model.toTag
import com.example.pmulabs.api.model.toUser
import com.example.pmulabs.api.model.toUserRemote
import com.example.pmulabs.room.models.Article
import com.example.pmulabs.room.models.Comment
import com.example.pmulabs.room.models.Tag
import com.example.pmulabs.room.models.User
import com.example.pmulabs.room.repository.OfflineArticleRepository
import com.example.pmulabs.room.repository.OfflineCommentRepository
import com.example.pmulabs.room.repository.OfflineTagRepository
import com.example.pmulabs.room.repository.OfflineUserRepository
import com.example.pmulabs.room.repository.UserRepository

class RestUserRepository(
    private val service: NewsPortalService,
    private val dbUserRepository: OfflineUserRepository,
    private val dbCommentRepository: OfflineCommentRepository,
    private val dbTagRepository: OfflineTagRepository,
    private val dbArticleRepository: OfflineArticleRepository
) : UserRepository {
    override suspend fun insertUser(user: User) {
        service.createUser(user.toUserRemote()).toUser()
    }

    override suspend fun getReport(startDate: Long, endDate: Long):List<ReportRemote>
    {
        return service.getReport(startDate,endDate)
    }

    override suspend fun updateUser(user: User) {
        user.id?.let { service.updateUser(it, user.toUserRemote()).toUser() }
    }

    override suspend fun deleteUser(user: User) {
        user.id?.let { service.deleteUser(it).toUser() }
    }

    override suspend fun getAllUsers(): List<User> {
        val existUsers = dbUserRepository.getAllUsers().associateBy { it.id }.toMutableMap()

        service.getUsers()
            .map { it.toUser() }
            .forEach { user ->
                val existUser = existUsers[user.id]
                if (existUser == null) {
                    dbUserRepository.insertUser(user)
                } else if (existUser != user) {
                    dbUserRepository.updateUser(user)
                }
                existUsers[user.id] = user
            }

        return existUsers.map { it.value }.sortedBy { it.id }
    }

    override suspend fun getUserById(idUser: Int?): User? =
        idUser?.let { service.getUser(it).toUser() }


    override suspend fun getUserComms(idUser: Int): List<Comment> {
        val existComments = dbUserRepository.getUserComms(idUser).associateBy { it.id }.toMutableMap()

        service.getUserComments(idUser)
            .map { it.toComment() }
            .forEach { comm ->
                if(comm.userId==idUser) {
                    val existComm = existComments[comm.id]
                    if (existComm == null) {
                        dbCommentRepository.insertComment(comm)
                    } else if (existComm != comm) {
                        dbCommentRepository.updateComment(comm)
                    }
                    existComments[comm.id] = comm
                }
            }

        return existComments.map { it.value }.sortedBy { it.id }
    }

    override suspend fun getUserArticles(idUser: Int): List<Article> {
        val existArticles = dbUserRepository.getUserArticles(idUser).associateBy { it.id }.toMutableMap()

        service.getUserArticles(idUser)
            .map { it.toArticle() }
            .forEach { art ->
                if(art.userId==idUser) {
                    val existArt = existArticles[art.id]
                    if (existArt == null) {
                        dbArticleRepository.insertArticle(art)
                    } else if (existArt != art) {
                        dbArticleRepository.updateArticle(art)
                    }
                    existArticles[art.id] = art
                }
            }

        return existArticles.map { it.value }.sortedBy { it.id }
    }

    override suspend fun getUserTags(idUser: Int): List<Tag> {
        val existTags = dbUserRepository.getUserTags(idUser).associateBy { it.id }.toMutableMap()

        service.getUserTags(idUser)
            .map { it.toTag() }
            .forEach { tag ->
                if(tag.userId==idUser) {
                    val existTag = existTags[tag.id]
                    if (existTag == null) {
                        dbTagRepository.insertTag(tag)
                    } else if (existTag != tag) {
                        dbTagRepository.updateTag(tag)
                    }
                    existTags[tag.id] = tag
                }
            }

        return existTags.map { it.value }.sortedBy { it.id }
    }
}