package com.example.pmulabs.room

import android.content.Context
import com.example.pmulabs.api.NewsPortalService
import com.example.pmulabs.api.repository.RestArticleRepository
import com.example.pmulabs.api.repository.RestCommentRepository
import com.example.pmulabs.api.repository.RestTagRepository
import com.example.pmulabs.api.repository.RestUserRepository
import com.example.pmulabs.room.database.NewsPortalDatabase
import com.example.pmulabs.room.repository.OfflineArticleRepository
import com.example.pmulabs.room.repository.OfflineCommentRepository
import com.example.pmulabs.room.repository.OfflineRemoteKeyRepository
import com.example.pmulabs.room.repository.OfflineTagRepository
import com.example.pmulabs.room.repository.OfflineUserRepository

interface AppContainer {
    val userRestRepository: RestUserRepository
    val tagRestRepository: RestTagRepository
    val articleRestRepository: RestArticleRepository
    val commentRestRepository: RestCommentRepository
    companion object {
        const val TIMEOUT = 5000L
        const val LIMIT = 10
    }
}

class AppDataContainer(private val context: Context) : AppContainer {
    private val userRepository: OfflineUserRepository by lazy {
        OfflineUserRepository(NewsPortalDatabase.getInstance(context).userDao())
    }
    private val articleRepository: OfflineArticleRepository by lazy {
        OfflineArticleRepository(NewsPortalDatabase.getInstance(context).articleDao())
    }
    private val tagRepository: OfflineTagRepository by lazy {
        OfflineTagRepository(NewsPortalDatabase.getInstance(context).tagDao())
    }
    private val commentRepository: OfflineCommentRepository by lazy {
        OfflineCommentRepository(NewsPortalDatabase.getInstance(context).commentDao())
    }
    private val remoteKeyRepository: OfflineRemoteKeyRepository by lazy {
        OfflineRemoteKeyRepository(NewsPortalDatabase.getInstance(context).remoteKeysDao())
    }
    override val userRestRepository: RestUserRepository by lazy {
        RestUserRepository(
            NewsPortalService.getInstance(),
            userRepository,
            commentRepository,
            tagRepository,
            articleRepository
        )
    }
    override val tagRestRepository: RestTagRepository by lazy {
        RestTagRepository(
            NewsPortalService.getInstance(),
            tagRepository,
            remoteKeyRepository,
            NewsPortalDatabase.getInstance(context)
        )
    }
    override val articleRestRepository: RestArticleRepository by lazy {
        RestArticleRepository(
            NewsPortalService.getInstance(),
            articleRepository,
            remoteKeyRepository,
            tagRestRepository,
            NewsPortalDatabase.getInstance(context)
        )
    }
    override val commentRestRepository: RestCommentRepository by lazy {
        RestCommentRepository(
            NewsPortalService.getInstance(),
            commentRepository,
            remoteKeyRepository,
            articleRestRepository,
            NewsPortalDatabase.getInstance(context)
        )
    }
}