package com.example.pmulabs.room.repository

import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.PagingData
import androidx.paging.PagingSource
import com.example.pmulabs.room.dao.ArticleDao
import com.example.pmulabs.room.models.Article
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first

class OfflineArticleRepository(private val articleDao: ArticleDao) : ArticleRepository {
    override suspend fun insertArticle(article: Article) = articleDao.insert(article)
    override suspend fun updateArticle(article: Article) = articleDao.update(article)
    override suspend fun deleteArticle(article: Article) = articleDao.delete(article)
    override suspend fun getAllArticles(): List<Article> = articleDao.getAll()
    override suspend fun getArticleById(idArticle: Int): Article? = articleDao.getArticleById(idArticle).first()
    override fun getArticles(): Flow<PagingData<Article>> = Pager(
        config = PagingConfig(
            pageSize = 4,
            enablePlaceholders = false
        ),
        pagingSourceFactory = articleDao::getArticles
    ).flow
    fun getAllArticlesPagingSource(): PagingSource<Int, Article> = articleDao.getArticles()

    suspend fun clearArticles() = articleDao.deleteAll()

    suspend fun insertArticles(articles: List<Article>) =
        articleDao.insert(*articles.toTypedArray())
}