package com.example.pmulabs.room.repository

import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.PagingData
import androidx.paging.PagingSource
import com.example.pmulabs.room.dao.CommentDao
import com.example.pmulabs.room.models.Comment
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first

class OfflineCommentRepository(private val commentDao: CommentDao) : CommentRepository {
    override suspend fun insertComment(comment: Comment) = commentDao.insert(comment)
    override suspend fun updateComment(comment: Comment) = commentDao.update(comment)
    override suspend fun deleteComment(comment: Comment) = commentDao.delete(comment)
    override suspend fun getAllComments(): List<Comment> = commentDao.getAll()
    override suspend fun getCountComment(idArticle : Int?) : Int = commentDao.getCountComment(idArticle)
    override fun getComments(): Flow<PagingData<Comment>> = Pager(
        config = PagingConfig(
            pageSize = 20,
            enablePlaceholders = false
        ),
        pagingSourceFactory =commentDao::getComments
    ).flow

    fun getAllCommentsPagingSource(): PagingSource<Int, Comment> = commentDao.getComments()

    suspend fun clearComments() = commentDao.deleteAll()

    override suspend fun getUser(commId: Int) = commentDao.getUser(commId).first()

    suspend fun insertComments(comments: List<Comment>) {
        commentDao.insert(*comments.toTypedArray())
    }
}

