package com.example.pmulabs.room.repository

import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.PagingData
import androidx.paging.PagingSource
import com.example.pmulabs.room.AppContainer
import com.example.pmulabs.room.dao.TagDao
import com.example.pmulabs.room.models.Tag
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first

class OfflineTagRepository(private val tagDao: TagDao) : TagRepository {
    override suspend fun insertTag(tag: Tag) = tagDao.insert(tag)
    override suspend fun updateTag(tag: Tag) = tagDao.update(tag)
    override suspend fun deleteTag(tag: Tag) = tagDao.delete(tag)
    override suspend fun getAllTags(): List<Tag> = tagDao.getAll()
    override suspend fun getTagById(idTag: Int): Tag? = tagDao.getTagById(idTag).first()
    override suspend fun getTagByName(nameTag: String): Tag? = tagDao.getTagByName(nameTag).first()
    override fun getTags(): Flow<PagingData<Tag>> = Pager(
        config = PagingConfig(
            pageSize = AppContainer.LIMIT,
            enablePlaceholders = false
        ),
        pagingSourceFactory = tagDao::getTags
    ).flow
    fun getAllTagsPagingSource(): PagingSource<Int, Tag> = tagDao.getTags()

    suspend fun clearTags() = tagDao.deleteAll()

    suspend fun insertTags(tags: List<Tag>) =
        tagDao.insert(*tags.toTypedArray())

}