package com.example.pmulabs.screensMobile

import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonColors
import androidx.compose.material3.DatePicker
import androidx.compose.material3.DatePickerColors
import androidx.compose.material3.DisplayMode
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.material3.rememberDatePickerState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.navigation.NavController
import com.example.pmulabs.api.model.ReportRemote
import com.example.pmulabs.designElem.elem.BackButton
import com.example.pmulabs.viewModels.AppViewModelProvider
import com.example.pmulabs.viewModels.ReportViewModel
import kotlinx.coroutines.launch
import java.text.SimpleDateFormat
import java.util.Date

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun ReportScreen(navController: NavController?, viewModel: ReportViewModel = viewModel(factory = AppViewModelProvider.Factory))
{
    val dateStateStart = rememberDatePickerState(initialDisplayMode = DisplayMode.Input)
    val dateStateEnd = rememberDatePickerState(initialDisplayMode = DisplayMode.Input)

    val coroutineScope = rememberCoroutineScope()
    val reportResultPageState = viewModel.reportResultPageUiState
    LazyColumn(
        contentPadding = PaddingValues(top=75.dp, bottom = 70.dp, start = 10.dp,end=10.dp),
        verticalArrangement = Arrangement.spacedBy(15.dp),
        modifier = Modifier
            .background(Color.White)
            .fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally
    )
    {
        item{
            BackButton(modifier=Modifier
                .clickable {
                    reportResultPageState.resReport= emptyList()
                    navController?.popBackStack()
                })
        }
        item {
            DatePicker(
                title = {
                    Text(
                        text = "Start Date",
                        style = MaterialTheme.typography.headlineLarge
                    )
                },
                state = dateStateStart,
                colors= DatePickerColors(
                    containerColor = Color.White,
                    currentYearContentColor=Color(0xff423a99),
                    dateTextFieldColors = TextFieldDefaults.textFieldColors(
                        Color(0xff423a99),
                        containerColor = Color.White,
                        focusedLabelColor = Color(0xff423a99),
                        focusedPlaceholderColor = Color(0xff423a99),
                        focusedTrailingIconColor = Color(0xff423a99)
                    ),
                    dayContentColor=Color(0xff423a99),
                    dayInSelectionRangeContainerColor=Color(0xff423a99),
                    dayInSelectionRangeContentColor=Color(0xff423a99),
                    disabledDayContentColor=Color(0xff423a99),
                    disabledSelectedDayContainerColor=Color(0xff423a99),
                    disabledSelectedDayContentColor=Color(0xff423a99),
                    disabledSelectedYearContainerColor=Color(0xff423a99),
                    disabledSelectedYearContentColor=Color(0xff423a99),
                    disabledYearContentColor=Color(0xff423a99),
                    dividerColor=Color(0xff423a99),
                    headlineContentColor=Color(0xff423a99),
                    navigationContentColor=Color(0xff423a99),
                    selectedDayContainerColor=Color(0xff423a99),
                    selectedDayContentColor=Color.White,
                    selectedYearContainerColor=Color(0xff423a99),
                    selectedYearContentColor=Color.White,
                    subheadContentColor=Color(0xff423a99),
                    titleContentColor=Color(0xff423a99),
                    todayContentColor=Color(0xff423a99),
                    todayDateBorderColor=Color(0xff423a99),
                    weekdayContentColor=Color(0xff423a99),
                    yearContentColor=Color(0xff423a99)
                ),
            )
            val selectedDateStart = dateStateStart.selectedDateMillis
            if (selectedDateStart != null) {
                val resultDate = selectedDateStart
                viewModel.onUpdate(viewModel.reportPageUiState.reportDetails.copy(startDate = resultDate))
            } else {
                viewModel.onUpdate(viewModel.reportPageUiState.reportDetails.copy(startDate = 0))
            }


            DatePicker(
                title ={
                    Text(
                    text = "End Date",
                    style = MaterialTheme.typography.headlineLarge
                )},
                state = dateStateEnd,
                colors= DatePickerColors(
                    containerColor = Color.White,
                    currentYearContentColor=Color(0xff423a99),
                    dateTextFieldColors = TextFieldDefaults.textFieldColors(
                        Color(0xff423a99),
                        containerColor = Color.White,
                        focusedLabelColor = Color(0xff423a99),
                        focusedPlaceholderColor = Color(0xff423a99),
                        focusedTrailingIconColor = Color(0xff423a99)
                    ),
                    dayContentColor=Color(0xff423a99),
                    dayInSelectionRangeContainerColor=Color(0xff423a99),
                    dayInSelectionRangeContentColor=Color(0xff423a99),
                    disabledDayContentColor=Color(0xff423a99),
                    disabledSelectedDayContainerColor=Color(0xff423a99),
                    disabledSelectedDayContentColor=Color(0xff423a99),
                    disabledSelectedYearContainerColor=Color(0xff423a99),
                    disabledSelectedYearContentColor=Color(0xff423a99),
                    disabledYearContentColor=Color(0xff423a99),
                    dividerColor=Color(0xff423a99),
                    headlineContentColor=Color(0xff423a99),
                    navigationContentColor=Color(0xff423a99),
                    selectedDayContainerColor=Color(0xff423a99),
                    selectedDayContentColor=Color.White,
                    selectedYearContainerColor=Color(0xff423a99),
                    selectedYearContentColor=Color.White,
                    subheadContentColor=Color(0xff423a99),
                    titleContentColor=Color(0xff423a99),
                    todayContentColor=Color(0xff423a99),
                    todayDateBorderColor=Color(0xff423a99),
                    weekdayContentColor=Color(0xff423a99),
                    yearContentColor=Color(0xff423a99)
                ),
            )
            val selectedDateEnd = dateStateEnd.selectedDateMillis
            if (selectedDateEnd != null) {
                val resultDate = selectedDateEnd
                viewModel.onUpdate(viewModel.reportPageUiState.reportDetails.copy(endDate = resultDate))
            } else {
                viewModel.onUpdate(viewModel.reportPageUiState.reportDetails.copy(endDate = 0))
            }

            Spacer(modifier = Modifier.height(16.dp))
            Button(
                colors = ButtonColors(
                    containerColor = Color(0xff423a99),
                    disabledContainerColor = Color(0xff423a99),
                    contentColor = Color.White,
                    disabledContentColor = Color.White
                ),
                onClick = {
                    coroutineScope.launch { viewModel.getReport() }
                },
                enabled = viewModel.reportPageUiState.isEntryValid,
                modifier = Modifier
                    .fillMaxWidth(0.5f)
                    .height(50.dp)
            ) {
                Text(text = "Get Report", fontSize = 20.sp)
            }
            Spacer(modifier = Modifier.height(32.dp))
            Text(
                color = Color(0xff423a99),
                text = "Result",
                style = MaterialTheme.typography.headlineLarge
            )
            TableScreen(reportData = reportResultPageState.resReport)
        }
    }
}

@Composable
fun ItemCell(
    title: String,
    commentCount: Int,
    category: String,
    userName: String,
    date: String
) {
    Box(
        modifier = Modifier
            .width(400.dp)
            .padding(8.dp)
            .clip(shape = RoundedCornerShape(5.dp))
            .background(Color(0xffdbdbf1))

    ) {
        Column(Modifier.padding(16.dp)) {
            Text(
                text = "Заголовок: $title",
                color = Color(0xff423a99),
                style = TextStyle(fontSize = 14.sp, fontWeight = FontWeight.Bold),
                modifier = Modifier.padding(bottom = 8.dp)
            )
            Text(
                text = "Кол-во комментариев: $commentCount",
                color = Color(0xff423a99),
                style = TextStyle(fontSize = 14.sp, fontWeight = FontWeight.Bold),
                modifier = Modifier.padding(bottom = 8.dp)
            )
            Text(
                text = "Категория: $category",
                color = Color(0xff423a99),
                style = TextStyle(fontSize = 14.sp, fontWeight = FontWeight.Bold),
                modifier = Modifier.padding(bottom = 8.dp)
            )
            Text(
                text = "Автор: $userName",
                color = Color(0xff423a99),
                style = TextStyle(fontSize = 14.sp, fontWeight = FontWeight.Bold),
                modifier = Modifier.padding(bottom = 8.dp)
            )
            Text(
                text = "Дата публикации: $date",
                color = Color(0xff423a99),
                style = TextStyle(fontSize = 14.sp, fontWeight = FontWeight.Bold),
                modifier = Modifier.padding(bottom = 8.dp)
            )
        }
    }
}


@Composable
fun TableScreen(reportData: List<ReportRemote>) {

    val column1Weight = .8f // 30%
    val column2Weight = 1f // 30%
    val formatter = SimpleDateFormat("dd-MMMM-YY")

    Column(
        Modifier
            .padding(16.dp)) {

        // Here are all the lines of your table.
        reportData.forEach {
            val (title, publishDate, tagName, userName,comments) = it
            Row(Modifier.fillMaxWidth()) {
                ItemCell(
                    title,
                    comments,
                    tagName,
                    userName,
                    formatter.format(Date(publishDate)).toString()
                )
            }
        }
    }
}