package com.example.pmulabs.viewModels

import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.ViewModel
import com.example.pmulabs.api.model.ReportRemote
import com.example.pmulabs.room.repository.UserRepository

class ReportViewModel(
    private val userRepository: UserRepository
) : ViewModel() {
    var reportPageUiState by mutableStateOf(ReportPageUiState())
        private set

    var reportResultPageUiState by mutableStateOf(ReportResultPageUiState())
        private set

    fun onUpdate(reportDetails: ReportDetails)
    {
        reportPageUiState = ReportPageUiState(reportDetails = reportDetails,isEntryValid = validateInput(reportDetails))
    }
    private fun validateInput(uiState: ReportDetails = reportPageUiState.reportDetails): Boolean {
        return with(uiState) {
            startDate >=0
                    && endDate >=0
                    && startDate < endDate
        }
    }
    suspend fun getReport(){
        val res = userRepository.getReport(reportPageUiState.reportDetails.startDate, reportPageUiState.reportDetails.endDate)
        reportResultPageUiState = ReportResultPageUiState(res)
    }
}

data class ReportDetails(
    val startDate: Long = 0,
    val endDate: Long = 0
)
data class ReportPageUiState(
    val reportDetails: ReportDetails = ReportDetails(),
    val isEntryValid: Boolean = false
)

data class ReportResultPageUiState(
    var resReport:List<ReportRemote> = emptyList()
)